/***************************************************************************\
*                                                                           *
* BuilderX Version 4.0                                                      *
*                                                                           *
* Copyright (C) 1999 Xironic. All Rights Reserved.                          *
*                                                                           *
\***************************************************************************/


//---------------------------------------------------------------------------
#ifndef BX3DObjectH
#define BX3DObjectH

#include "BX3DFrame.h"
#include "BX3DLight.h"

enum TMShapeType{stFile,stSphere,stPrism,stCube};
enum TFileFormat{ffBinary,ffCompressed,ffText};
enum TMShadeMode{tsGouraud,tsFlat,tsPhong};
enum TMFillMode{tfSolid,tfPoints,tfWireframe};
enum TWrapType{twFlat,twCylinder,twSphere,twChrome};

typedef void __fastcall (__closure *TCollisionEvent)(System::TObject* Sender, System::TObject* CollisionObject);

class TBX3DObject;
class TBX3DTexture;

//TBX3DWrap
//---------------------------------------------------------------------------
class PACKAGE TBX3DWrap : public TPersistent
{
  private:
    TBX3DObject* OwnerObject;
    float FOriginX;
    float FOriginY;
    float FOriginZ;
    float FDirX;
    float FDirY;
    float FDirZ;
    float FUpX;
    float FUpY;
    float FUpZ;
    float FTextureOriginU;
    float FTextureOriginV;
    float FTextureScaleU;
    float FTextureScaleV;
    void __fastcall SetOriginX(float);
    void __fastcall SetOriginY(float);
    void __fastcall SetOriginZ(float);
    void __fastcall SetDirX(float);
    void __fastcall SetDirY(float);
    void __fastcall SetDirZ(float);
    void __fastcall SetUpX(float);
    void __fastcall SetUpY(float);
    void __fastcall SetUpZ(float);
    void __fastcall SetTextureOriginU(float);
    void __fastcall SetTextureOriginV(float);
    void __fastcall SetTextureScaleU(float);
    void __fastcall SetTextureScaleV(float);

    TBX3DCustomFrame* FRefFrame;
    void __fastcall SetRefFrame(TBX3DCustomFrame*);

    TWrapType FType;
    void __fastcall SetType(TWrapType);

    LPDIRECT3DRMWRAP FIDirect3DWrap;
    LPDIRECT3DRMWRAP __fastcall GetIDirect3DWrap();
  public:
    virtual void __fastcall Assign(TPersistent* Source);
    void __fastcall Change();
    __fastcall TBX3DWrap(TBX3DObject*);
    __fastcall ~TBX3DWrap();

    __property LPDIRECT3DRMWRAP IDirect3DWrap={read=GetIDirect3DWrap,write=FIDirect3DWrap};
  __published:
    __property TBX3DCustomFrame* RefFrame={read=FRefFrame,write=SetRefFrame};
    __property float OriginX={read=FOriginX,write=SetOriginX};
    __property float OriginY={read=FOriginY,write=SetOriginY};
    __property float OriginZ={read=FOriginZ,write=SetOriginZ};
    __property float DirX={read=FDirX,write=SetDirX};
    __property float DirY={read=FDirY,write=SetDirY};
    __property float DirZ={read=FDirZ,write=SetDirZ};
    __property float UpX={read=FUpX,write=SetUpX};
    __property float UpY={read=FUpY,write=SetUpY};
    __property float UpZ={read=FUpZ,write=SetUpZ};
    __property float TextureOriginU={read=FTextureOriginU,write=SetTextureOriginU};
    __property float TextureOriginV={read=FTextureOriginV,write=SetTextureOriginV};
    __property float TextureScaleU={read=FTextureScaleU,write=SetTextureScaleU,default=1};
    __property float TextureScaleV={read=FTextureScaleV,write=SetTextureScaleV,default=1};
    __property TWrapType Type={read=FType,write=SetType};
};

//TBX3DDecal
//---------------------------------------------------------------------------
class PACKAGE TBX3DDecal : public TPersistent
{
  private:
    TBX3DTexture* OwnerTexture;

    bool FTransparent;
    void __fastcall SetTransparent(bool);
    TColor FTransparentColor;
    void __fastcall SetTransparentColor(TColor);
    bool FScale;
    void __fastcall SetScale(bool);
    float FWidth;
    void __fastcall SetWidth(float);
    float FHeight;
    void __fastcall SetHeight(float);
    LONG FOrginX;
    void __fastcall SetOrginX(LONG);
    LONG FOrginY;
    void __fastcall SetOrginY(LONG);

  public:
    void __fastcall Change();
    __fastcall TBX3DDecal(TBX3DTexture*);

  __published:
    __property bool Scale={read=FScale,write=SetScale};
    __property float Width={read=FWidth,write=SetWidth};
    __property float Height={read=FHeight,write=SetHeight};
    __property LONG OrginX={read=FOrginX,write=SetOrginX};
    __property LONG OrginY={read=FOrginY,write=SetOrginY};
    __property bool Transparent={read=FTransparent,write=SetTransparent};
    __property TColor TransparentColor={read=FTransparentColor,write=SetTransparentColor};
};

//TBX3DQuality
//---------------------------------------------------------------------------
class PACKAGE TBX3DQuality : public TPersistent
{
  private:
    TBX3DObject* OwnerObject;
    TMShadeMode FShadeMode;
    bool FLightMode;
    TMFillMode FFillMode;
    void __fastcall SetMShadeMode(TMShadeMode);
    void __fastcall SetMLightMode(bool);
    void __fastcall SetMFillMode(TMFillMode);
    TMShadeMode __fastcall GetMShadeMode();
    bool __fastcall GetMLightMode();
    TMFillMode __fastcall GetMFillMode();

  public:
    virtual void __fastcall Assign(TPersistent* Source);
    void __fastcall Change();
    __fastcall TBX3DQuality(TBX3DObject*);

  __published:
    __property TMShadeMode ShadeMode={read=GetMShadeMode,write=SetMShadeMode};
    __property bool LightMode={read=GetMLightMode,write=SetMLightMode,default=true};
    __property TMFillMode FillMode={read=GetMFillMode,write=SetMFillMode};
};

//TBX3DFace
//---------------------------------------------------------------------------
class PACKAGE TBX3DFace : public TPersistent
{
  private:
  public:
   DWORD FVertexCount;
   D3DVECTOR* FVertices;
   DWORD FNormalCount;
   D3DVECTOR* FNormals;
   DWORD FFaceDataSize;
   DWORD* FFaceData;
    __fastcall TBX3DFace(DWORD,D3DVECTOR*,DWORD,D3DVECTOR*,DWORD,DWORD*);
    __fastcall ~TBX3DFace();
  __published:
};

class TBX3DMesh;

//TBX3DVertex
//---------------------------------------------------------------------------
class PACKAGE TBX3DVertex : public TPersistent
{
  private:
   TBX3DMesh* OwnerMesh;
   DWORD FIndex;
   float FX;
   float FY;
   float FZ;
   void __fastcall SetX(float);
   float __fastcall GetX();
   void __fastcall SetY(float);
   float __fastcall GetY();
   void __fastcall SetZ(float);
   float __fastcall GetZ();
   void __fastcall Change();

  public:
    void __fastcall SetXYZ(float X,float Y,float Z);
    __fastcall TBX3DVertex(TBX3DMesh* OwnerMesh,DWORD Index);
    __fastcall TBX3DVertex(float X,float Y,float Z);
    __property float X={read=GetX,write=SetX};
    __property float Y={read=GetY,write=SetY};
    __property float Z={read=GetZ,write=SetZ};

  __published:
};

//TBX3DShape
//---------------------------------------------------------------------------
class PACKAGE TBX3DShape : public TPersistent
{
  protected:
    TMShapeType FType;
    TBX3DMesh* OwnerMesh;
    TBX3DFace* FFace;
    virtual void __fastcall Create()=0;

  public:
    void __fastcall Set();
    __fastcall TBX3DShape(TBX3DMesh*);
    __fastcall ~TBX3DShape();

  __published:
};

//TBX3DPrism
//---------------------------------------------------------------------------
class PACKAGE TBX3DPrism : public TBX3DShape
{
  private:
    float FDepth;
    void __fastcall SetDepth(float);
    float FAperture;
    void __fastcall SetAperture(float);
    int FVertices;
    void __fastcall SetVertices(int);
    void __fastcall Create();
    void __fastcall CreateFaces_PolyForm();
    void __fastcall CreateFaces_PolyForm2();
    void __fastcall CreateFaces_PolyFormG();
    void __fastcall CreateFaces_PolyFormG2();

  public:
    __fastcall TBX3DPrism(TBX3DMesh*);

  __published:
    __property float Depth={read=FDepth,write=SetDepth};
    __property float Aperture={read=FAperture,write=SetAperture};
    __property int Vertices={read=FVertices,write=SetVertices,default=3};
};

//TBX3DSphere
//---------------------------------------------------------------------------
class PACKAGE TBX3DSphere : public TBX3DShape
{
  private:
    int FEdges;
    void __fastcall SetEdges(int);
    int FWalls;
    void __fastcall SetWalls(int);
    void __fastcall Create();
    void __fastcall CreateFaces_Sphere();
    void __fastcall CreateFaces_Sphere2();
    
  public:
    __fastcall TBX3DSphere(TBX3DMesh*);

  __published:
    __property int Edges={read=FEdges,write=SetEdges,default=15};
    __property int Walls={read=FWalls,write=SetWalls,default=15};
};

//TBX3DCube
//---------------------------------------------------------------------------
class PACKAGE TBX3DCube : public TBX3DShape
{
  private:
    int FGrid;
    void __fastcall SetGrid(int);
    void __fastcall Create();

  public:
    __fastcall TBX3DCube(TBX3DMesh*);

  __published:
    __property int Grid={read=FGrid,write=SetGrid,default=3};
};

//TBX3DFile
//---------------------------------------------------------------------------
class PACKAGE TBX3DFile : public TPersistent
{
  private:
    TBX3DMesh* OwnerMesh;
    AnsiString FFileName;
    void __fastcall SetFileName(AnsiString);
    TFileFormat FFormat;

  public:
    TFileFormat __fastcall GetFileFormat(AnsiString);
    void __fastcall LoadFromFile(AnsiString);
    void __fastcall SaveToFile(AnsiString);
    __fastcall TBX3DFile(TBX3DMesh*);

  __published:
    __property AnsiString FileName={read=FFileName,write=SetFileName};
    __property TFileFormat Format={read=FFormat,write=FFormat};
};

//TBX3DMesh
//---------------------------------------------------------------------------
class PACKAGE TBX3DMesh : public TPersistent
{
  private:
    TBX3DCube* FCube;
    TBX3DPrism* FPrism;
    TBX3DSphere* FSphere;
    TBX3DFile* FFile;

    bool FGenerateNormals;
    void __fastcall SetGenerateNormals(bool);

    TMShapeType FActiveShape;
    void __fastcall SetActiveShape(TMShapeType);

    TBX3DObject* OwnerObject;

    void __fastcall FixMesh();

    DWORD FVertexCount;
    DWORD FNormalCount;
    D3DVECTOR* FVertices;
    D3DVECTOR* FNormals;
    DWORD FFace_data_size;
    DWORD* FFace_data;
    void __fastcall SetChange();
    DWORD* __fastcall SearchFace(DWORD);
    void __fastcall MakeFaceData(DWORD* vc,D3DVECTOR** v,DWORD* nc,D3DVECTOR** n,DWORD* ds,DWORD** d,D3DVECTOR* sv,D3DVECTOR* sn,DWORD* sd);

    LPDIRECT3DRMMESHBUILDER3 FIDirect3DMeshBuilder;
    LPDIRECT3DRMMESHBUILDER3 __fastcall GetIDirect3DMeshBuilder();

  public:
    virtual void __fastcall Assign(TPersistent* Source);
    void __fastcall ChangeColor();
    void __fastcall UpdateMeshVisible();
    void __fastcall Release();
    void __fastcall SetVisible(bool);
    void __fastcall Change();
    void __fastcall ChangeShape();

    void __fastcall SetVertexXYZ(DWORD Index,float X,float Y,float Z);
    void __fastcall SetVertexX(DWORD Index,float X);
    void __fastcall SetVertexY(DWORD Index,float Y);
    void __fastcall SetVertexZ(DWORD Index,float Z);
    D3DVECTOR __fastcall GetVertexXYZ(DWORD Index);
    float GetVertexX(DWORD Index);
    float GetVertexY(DWORD Index);
    float GetVertexZ(DWORD Index);
    bool __fastcall GetFace(DWORD,TBX3DFace**);
    void __fastcall AddFace(TBX3DFace*);
    void __fastcall SetFace(TBX3DFace*);
    void __fastcall DeleteFace(DWORD);
    __fastcall TBX3DMesh(TBX3DObject*);
    __fastcall TBX3DMesh(TBX3DObject*,TBX3DMesh*);
    __fastcall ~TBX3DMesh();
    void __fastcall Create();

    __property LPDIRECT3DRMMESHBUILDER3 IDirect3DMeshBuilder={read=GetIDirect3DMeshBuilder,write=FIDirect3DMeshBuilder};
  __published:
    __property TBX3DFile* File={read=FFile,write=FFile};
    __property TBX3DCube* Cube={read=FCube,write=FCube};
    __property TBX3DSphere* Sphere={read=FSphere,write=FSphere};
    __property TBX3DPrism* Prism={read=FPrism,write=FPrism};
    __property bool GenerateNormals={read=FGenerateNormals,write=SetGenerateNormals};
    __property TMShapeType ActiveShape={read=FActiveShape,write=SetActiveShape};
};

//TBX3DShadow
//---------------------------------------------------------------------------
class PACKAGE TBX3DShadow : public TPersistent
{
  private:
    TBX3DObject* OwnerObject;
    TBX3DLight* FLight;
    void __fastcall SetLight(TBX3DLight*);
    float FPlaneX;
    float FPlaneY;
    float FPlaneZ;
    float FNormalX;
    float FNormalY;
    float FNormalZ;
    void __fastcall SetPlaneX(float);
    void __fastcall SetPlaneY(float);
    void __fastcall SetPlaneZ(float);
    void __fastcall SetNormalX(float);
    void __fastcall SetNormalY(float);
    void __fastcall SetNormalZ(float);
    float __fastcall GetPlaneX();
    float __fastcall GetPlaneY();
    float __fastcall GetPlaneZ();
    float __fastcall GetNormalX();
    float __fastcall GetNormalY();
    float __fastcall GetNormalZ();
    void __fastcall NoShadow();
    void __fastcall ChangeParam();
    LPDIRECT3DRMSHADOW2 FIDirect3DShadow;
  public:
    virtual void __fastcall Assign(TPersistent* Source);
    __fastcall TBX3DShadow(TBX3DObject*);
    __fastcall ~TBX3DShadow();
    void __fastcall Change();

    __property LPDIRECT3DRMSHADOW2 IDirect3DShadow={read=FIDirect3DShadow,write=FIDirect3DShadow};
  __published:
    __property float PlaneX={read=GetPlaneX,write=SetPlaneX};
    __property float PlaneY={read=GetPlaneY,write=SetPlaneY};
    __property float PlaneZ={read=GetPlaneZ,write=SetPlaneZ};
    __property float NormalX={read=GetNormalX,write=SetNormalX};
    __property float NormalY={read=GetNormalY,write=SetNormalY};
    __property float NormalZ={read=GetNormalZ,write=SetNormalZ};
    __property TBX3DLight* Light={read=FLight,write=SetLight};
};

//TBX3DScale
//---------------------------------------------------------------------------
class PACKAGE TBX3DScale : public TPersistent
{
  private:
    TBX3DObject* OwnerObject;
    float FX;
    float FY;
    float FZ;
    void __fastcall SetX(float);
    void __fastcall SetY(float);
    void __fastcall SetZ(float);

  public:
    virtual void __fastcall Assign(TPersistent* Source);
    void __fastcall Change();
    void __fastcall UnChange();
    __fastcall TBX3DScale(TBX3DObject*);
    __fastcall ~TBX3DScale();

  __published:
    __property float X={read=FX,write=SetX};
    __property float Y={read=FY,write=SetY};
    __property float Z={read=FZ,write=SetZ};
};

//TBX3DTexture
//---------------------------------------------------------------------------
class PACKAGE TBX3DTexture : public TPersistent
{
  private:
    TBX3DObject* OwnerObject;

    bool FUseMipMap;
    void __fastcall SetUseMipMap(bool);

    TBX3DDecal* FDecal;
    void __fastcall SetDecal(TBX3DDecal*);
    void __fastcall ChangeDecal();

    int FColors;
    void __fastcall SetColors(int);
    int __fastcall GetColors();

    DWORD FShades;
    void __fastcall SetShades(DWORD);
    DWORD __fastcall GetShades();

    AnsiString FFileName;
    void __fastcall SetFileName(AnsiString);

    LPDIRECT3DRMTEXTURE3 FIDirect3DTexture;
    LPDIRECT3DRMTEXTURE3 __fastcall GetIDirect3DTexture();
    void __fastcall SetIDirect3DTexture(LPDIRECT3DRMTEXTURE3);

  public:
    virtual void __fastcall Assign(TPersistent* Source);
    __fastcall TBX3DTexture(TBX3DObject*);
    __fastcall ~TBX3DTexture();
    void __fastcall Change();

    __property LPDIRECT3DRMTEXTURE3 IDirect3DTexture={read=GetIDirect3DTexture};
  __published:
    __property bool UseMipMap={read=FUseMipMap,write=SetUseMipMap};
    __property DWORD Shades={read=GetShades,write=SetShades};
    __property int Colors={read=GetColors,write=SetColors};
    __property TBX3DDecal* Decal={read=FDecal,write=SetDecal};
    __property AnsiString FileName={read=FFileName,write=SetFileName};
};

//TBX3DMaterial
//---------------------------------------------------------------------------
class PACKAGE TBX3DMaterial : public TPersistent
{
  private:
    TBX3DObject* OwnerObject;

    TColor FEmission;
    void __fastcall SetEmission(TColor);
    TColor __fastcall GetEmission();
    float FPower;
    void __fastcall SetPower(float);
    float __fastcall GetPower();
    TColor FSpecular;
    void __fastcall SetSpecular(TColor);
    TColor __fastcall GetSpecular();
    TColor FAmbient;
    void __fastcall SetAmbient(TColor);
    TColor __fastcall GetAmbient();

    LPDIRECT3DRMMATERIAL2 FIDirect3DMaterial;
    LPDIRECT3DRMMATERIAL2 __fastcall GetIDirect3DMaterial();
    void __fastcall SetIDirect3DMaterial(LPDIRECT3DRMMATERIAL2);

  public:
    virtual void __fastcall Assign(TPersistent* Source);
    __fastcall TBX3DMaterial(TBX3DObject*);
    __fastcall ~TBX3DMaterial();
    void __fastcall Change();

    __property LPDIRECT3DRMMATERIAL2 IDirect3DMaterial={read=GetIDirect3DMaterial,write=SetIDirect3DMaterial};
  __published:
    __property TColor Emission={read=GetEmission,write=SetEmission};
    __property float Power={read=GetPower,write=SetPower};
    __property TColor Specular={read=GetSpecular,write=SetSpecular};
    __property TColor Ambient={read=GetAmbient,write=SetAmbient};
};

//TBX3DObject
//---------------------------------------------------------------------------
class PACKAGE TBX3DObject : public TBX3DFrame
{
  private:
    bool FVisible;
    void __fastcall SetVisible(bool);

    float FColorAlpha;
    void __fastcall SetColorAlpha(float);

    TBX3DWrap* FWrap;
    void __fastcall SetWrap(TBX3DWrap*);
    void __fastcall ChangeWrapType();

    TBX3DQuality* FQuality;
    void __fastcall SetQuality(TBX3DQuality*);
    void __fastcall ChangeQualityMode();

    TBX3DShadow* FShadow;
    void __fastcall SetShadow(TBX3DShadow*);
    void __fastcall ChangeShadow();

    TBX3DScale* FScale;
    void __fastcall SetScale(TBX3DScale*);

    TBX3DTexture* FTexture;
    void __fastcall SetTexture(TBX3DTexture*);

    TBX3DMaterial* FMaterial;
    void __fastcall SetMaterial(TBX3DMaterial*);
    void __fastcall ChangeMaterial();

    TBX3DMesh* FMesh;
    void __fastcall SetMesh(TBX3DMesh*);
    void __fastcall ChangeMesh();

    bool FCheckCollision;
    TCollisionEvent FOnCollision;

    bool FPerspective;
    void __fastcall SetPerspective(bool);
    void __fastcall ChangePerspective();
    bool __fastcall GetPerspective();

    void __fastcall Init();

    void __fastcall SetScene(TBX3DFrame*);
    TBX3DFrame* __fastcall GetScene();
    void __fastcall ChangeColor();

  protected:
    virtual void __fastcall Create();
    void __fastcall Loaded(void);
    virtual void __fastcall DoOnMove();

  public:
    virtual void __fastcall Notification(TComponent* AComponent,TOperation Operation);
    D3DVECTOR BoxMax;
    D3DVECTOR BoxMin;

    void __fastcall MeshChanged();
    void __fastcall ChangeScale();
    void __fastcall ChangeTexture();
    void __fastcall UpdateTexture();
    void __fastcall Change();
    void __fastcall Collision(System::TObject*,System::TObject*);

//methods for client
    void __fastcall Clone(TBX3DObject**);
    __fastcall TBX3DObject(TComponent* Owner);
    __fastcall ~TBX3DObject();

  __published:
    __property bool Visible={read=FVisible,write=SetVisible,default=true};
    __property TBX3DMesh* Mesh={read=FMesh,write=SetMesh};
    __property float ColorAlpha={read=FColorAlpha,write=SetColorAlpha};
    __property TBX3DScale* Scale={read=FScale,write=SetScale};
    __property TCollisionEvent OnCollision={read=FOnCollision,write=FOnCollision};
    __property bool CheckCollision={read=FCheckCollision,write=FCheckCollision};
    __property bool Perspective={read=GetPerspective,write=SetPerspective,default=true};
    __property TColor Color={read=FColor,write=SetColor};
    __property TBX3DQuality* Quality={read=FQuality,write=SetQuality};
    __property TBX3DWrap* Wrap={read=FWrap,write=SetWrap};
    __property TBX3DShadow* Shadow={read=FShadow,write=SetShadow};
    __property TBX3DTexture* Texture={read=FTexture,write=SetTexture};
    __property TBX3DMaterial* Material={read=FMaterial,write=SetMaterial};
};
//---------------------------------------------------------------------------
#endif
